from .dataset.gsm8k import gsm8k_split
from .dataset.multiarith import multiarith_split
from .dataset.bigbenchhard import bbh_split
from .dataset.gsmhard import gsmhard_split
from .dataset.wikitablequestions import wikitq_split
from .dataset.strategyqa import strategyqa_split
from .dataset.arc import arc_split
from .dataset.bigbench import bigbench_split
from .dataset.aquarat import aquarat_split
from .dataset.gpqa import gpqa_split
from .dataset.svamp import svamp_split
from .dataset.csqa import csqa_split
import random
from .dataset.base import base_load_dataset

class LLMdataset:
    def __init__(
        self,
        dataset_name = None,
        subset = None
        ):
        self.dataset_name = dataset_name
        self.subset = subset
        self.keys_list, count_list, self.dataset = base_load_dataset(self.dataset_name, self.subset)
        print(f"Data types:{self.keys_list}")
        print(f"Number of data:{count_list}")


    def _split_dataset(self, data_type):
        if self.dataset_name == 'gsm8k':
            data_list, num_data = gsm8k_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        elif self.dataset_name == "ChilleD/MultiArith":
            data_list, num_data = multiarith_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        elif self.dataset_name == "lukaemon/bbh":
            data_list, num_data = bbh_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        elif self.dataset_name == "reasoning-machines/gsm-hard":
            data_list, num_data = gsmhard_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        elif self.dataset_name == "wikitablequestions":
            data_list, num_data = wikitq_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        elif self.dataset_name == "ChilleD/StrategyQA":
            data_list, num_data = strategyqa_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        elif self.dataset_name == "allenai/ai2_arc":
            data_list, num_data = arc_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        elif self.dataset_name == "tasksource/bigbench":
            data_list, num_data = bigbench_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        elif self.dataset_name == "aqua_rat":
            data_list, num_data = aquarat_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        elif self.dataset_name == "Idavidrein/gpqa":
            data_list, num_data = gpqa_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        elif self.dataset_name == "ChilleD/SVAMP":
            data_list, num_data = svamp_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        elif self.dataset_name == "tau/commonsense_qa":
            data_list, num_data = csqa_split(self.dataset, data_type, self.keys_list)
            return data_list, num_data
        else:
            raise ValueError(f"Unsupported dataset name: {self.dataset_name}")


    def dataloader(self, data_type=None, batch_size=1, seed=None, max_data=5):
        self.data_list, self.num_data = self._split_dataset(data_type)

        if seed is not None:
            random.seed(seed)

        if max_data is not None and max_data > self.num_data:
            raise ValueError(f"max_data ({max_data}) cannot be greater than the size of the dataset ({self.num_data}).")

        if max_data is not None:
            data_list = random.sample(self.data_list, max_data)
        else:
            data_list = self.dataset[:max_data]

        for i in range(0, len(data_list), batch_size):
            yield data_list[i:i + batch_size]





from .base import base_data_split

def svamp_split(dataset, data_type, keys_list):
    all_dataset, num_data = base_data_split(dataset, data_type, keys_list)
    data_list = [[example['Body'], example['Question'], example['Answer']] for example in all_dataset]
    return data_list, num_data



from datasets import load_dataset, concatenate_datasets

def base_load_dataset(dataset_name, subset):
    try:
        dataset = load_dataset(dataset_name, subset)
        keys_list = list(dataset.keys())
        count_list = []
        for i in range(len(keys_list)):
            num_dataset = len(dataset[keys_list[i]])
            count_list.append(num_dataset)
        return keys_list, count_list, dataset

    except Exception as e:
        print(f"Error occurred: {e}")
        return None


def base_data_split(dataset, data_type, keys_list):
    if data_type == 'all':
        concat_data_list = []
        for i in range(len(keys_list)):
            all_dataset = dataset[keys_list[i]]
            concat_data_list.append(all_dataset)
        all_dataset = concatenate_datasets(concat_data_list)
        num_data = len(all_dataset)
    else:
        all_dataset = dataset[data_type]
        num_data = len(all_dataset)

    return all_dataset, num_data
